@@ -537,21 +537,23 @@ void SciTEWin::ReadProperties() {
 	}
 }
 
-static FilePath GetSciTEPath(const FilePath &home) {
+namespace {
+
+FilePath GetSciTEPath(const FilePath &home) {
 	if (home.IsSet()) {
-		return FilePath(home);
+		return home;
 	} else {
-		GUI::gui_char path[MAX_PATH];
-		if (::GetModuleFileNameW(0, path, static_cast<DWORD>(std::size(path))) == 0)
-			return FilePath();
+		GUI::gui_char path[MAX_PATH+1]{};
+		if (::GetModuleFileNameW(0, path, MAX_PATH) == 0)
+			return {};
 		// Remove the SciTE.exe
-		GUI::gui_char *lastSlash = wcsrchr(path, pathSepChar);
-		if (lastSlash)
-			*lastSlash = '\0';
-		return FilePath(path);
+		const FilePath pathSciTE(path);
+		return pathSciTE.Directory();
 	}
 }
 
+}
+
 FilePath SciTEWin::GetDefaultDirectory() {
 	const GUI::gui_char *home = _wgetenv(GUI_TEXT("SciTE_HOME"));
 	return GetSciTEPath(home);
@@ -821,8 +823,12 @@ void CommandWorker::Initialise(bool resetToStart) noexcept {
 	outputScroll = 1;
 }
 
-void CommandWorker::Execute() {
-	pSciTE->ProcessExecute();
+void CommandWorker::Execute() noexcept {
+	try {
+		pSciTE->ProcessExecute();
+	} catch (...) {
+		pSciTE->OutputAppendEncodedStringSynchronised(L"Exception in thread.\r\n", 0);
+	}
 }
 
 void SciTEWin::ResetExecution() {
@@ -1236,7 +1242,7 @@ void SciTEWin::ShellExec(std::string_view cmd, std::string_view dir) {
 		s = strstr(mycmdLowered, ".com");
 	std::string cmdcopy(cmd);
 	char *mycmdcopy = &cmdcopy[0];
-	char *mycmd;
+	const char *mycmd;
 	char *mycmdEnd = nullptr;
 	if (s && ((*(s + 4) == '\0') || (*(s + 4) == ' '))) {
 		ptrdiff_t len_mycmd = s - mycmdLowered + 4;
