@@ -92,6 +92,7 @@ typedef void *HTHEME;
 #include "JobQueue.h"
 #include "Cookie.h"
 #include "Worker.h"
+#include "Utf8_16.h"
 #include "FileWorker.h"
 #include "MatchMarker.h"
 #include "Searcher.h"
@@ -126,7 +127,7 @@ public:
 
 	CommandWorker() noexcept;
 	void Initialise(bool resetToStart) noexcept;
-	void Execute() override;
+	void Execute() noexcept override;
 };
 
 class Dialog;
@@ -458,8 +459,4 @@ inline HMENU HmenuID(size_t id) noexcept {
 	return reinterpret_cast<HMENU>(id);
 }
 
-inline POINT *PointPointer(GUI::Point *pt) noexcept {
-	return reinterpret_cast<POINT *>(pt);
-}
-
 #endif
